/*
 *
 * VMMZ: Delta Encoding of Virtual-Machine Memory
 * Copyright (C) 2015-2016  James E. Fowler
 *
 * This file is a part of VMMZ.
 *
 * VMMZ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VMMZ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "vmmz.h"

void VMMZCopyFileInitialize(VMMZCopyFile *copy_file)
{
  if (copy_file != NULL)
    {
      QccStringMakeNull(copy_file->filename);
      copy_file->num_copies = 0;
      copy_file->num_allocated_copies = 0;
      copy_file->copies = NULL;
    }
}


void VMMZCopyFileFree(VMMZCopyFile *copy_file)
{
  if (copy_file != NULL)
    if (copy_file->copies != NULL)
      free(copy_file->copies);
}


int VMMZCopyFileAddCopy(VMMZCopyFile *copy_file,
                        const VMMZCopyEntry *copy)
{
  int return_value = 0;

  if (copy_file == NULL)
    goto Return;
  if (copy == NULL)
    goto Return;

  copy_file->num_copies++;
  if (copy_file->num_copies > copy_file->num_allocated_copies)
    {
      copy_file->num_allocated_copies += VMMZ_COPYFILE_REALLOC_SIZE;
      if ((copy_file->copies =
           (VMMZCopyEntry *)realloc((void *)copy_file->copies,
                                    sizeof(VMMZCopyEntry) *
                                    copy_file->num_allocated_copies)) == NULL)
        {
          QccErrorAddMessage("(VMMZCopyFileAddCopy): Error reallocating memory");
          goto Error;
        }
    }

  copy_file->copies[copy_file->num_copies - 1] = *copy;
  
  return_value = 0;
  goto Return;
 Error:
  return_value = 1;
 Return:
  return(return_value);
}


int VMMZCopyFileInsert(unsigned char *memory,
                       long memory_size,
                       VMMZCopyFile *copy_file,
                       const QccString mount_point)
{
  int return_value = 0;
  int copy = 0;
  VMMZPEImage pe_image;
  QccString fullpath;
  
  VMMZPEImageInitialize(&pe_image);
  
  if (memory == NULL)
    return(0);
  if (copy_file == NULL)
    return(0);

  QccStringSprintf(fullpath, "%s/%s",
                   mount_point,
                   copy_file->filename);

  if (VMMZPEImageRead(fullpath, &pe_image))
    {
      QccErrorAddMessage("(VMMZCopyFileInsert): Error calling VMMZPEImageRead()");
      goto Error;
    }
  
  for (copy = 0; copy < copy_file->num_copies; copy++)
    {
      if ((copy_file->copies[copy].source + copy_file->copies[copy].length) >
          pe_image.size)
        {
          QccErrorAddMessage("(VMMZCopyFileInsert): Invalid memcpy");
          goto Error;
        }
      if ((copy_file->copies[copy].destination +
           copy_file->copies[copy].length) > memory_size)
        {
          QccErrorAddMessage("(VMMZCopyFileInsert): Invalid memcpy");
          goto Error;
        }
      memcpy((void *)&memory[copy_file->copies[copy].destination],
             (void *)&pe_image.image[copy_file->copies[copy].source],
             (size_t)copy_file->copies[copy].length);
    }
  
  return_value = 0;
  goto Return;
 Error:
  return_value = 1;
 Return:
  VMMZPEImageFree(&pe_image);
  return(return_value);
}
