/*
 *
 * VMMZ: Delta Encoding of Virtual-Machine Memory
 * Copyright (C) 2015-2016  James E. Fowler
 *
 * This file is a part of VMMZ.
 *
 * VMMZ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VMMZ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef VMMZ_PE_H
#define VMMZ_PE_H

#include "vmmz_pe_defs.h"

#define XSTR(S) STR(S)
#define STR(S) #S

#define VMMZ_VMWARE_DEFAULT_VMDK_STR XSTR(VMMZ_VMWARE_DEFAULT_VMDK)
#define VMMZ_VMWARE_MOUNT_CMD_STR XSTR(VMMZ_VMWARE_MOUNT_CMD)
#define VMMZ_LOSETUP_CMD_STR XSTR(VMMZ_LOSETUP_CMD)
#define VMMZ_VOLATILITY_CMD_STR XSTR(VMMZ_VOLATILITY_CMD)
#define VMMZ_VOLATILITY_DEFAULT_PROFILE_STR XSTR(VMMZ_VOLATILITY_DEFAULT_PROFILE)
#define VMMZ_INSTALL_PATH_STR XSTR(VMMZ_INSTALL_PATH)
#define VMMZ_PARSE_MAGIC_NUM "VMMZ"
#define VMMZ_TMPDIR "/tmp/VMMZ_XXXXXX"

#include "vmmz_petable.h"
#include "vmmz_pagetable.h"
#include "vmmz_processtable.h"
#include "vmmz_copytable.h"
#include "vmmz_peimage.h"

typedef struct
{
  QccString name;
  unsigned int virtual_size;
  unsigned int virtual_address;
  unsigned int raw_size;
  unsigned int raw_ptr;
} VMMZSection;


int VMMZGetFileSize(const QccString filename, long *filesize);
int VMMZGetPETable(const QccString baseline_filename,
                   const QccString current_vmem_filename,
                   const QccString vmdk_filename,
                   const QccString volatility_profile,
                   VMMZPETable *pe_table,
                   VMMZProcessTable *process_table,
                   VMMZCopyTable *copy_table);
int VMMZInsertPEs(unsigned char *memory,
                  long memory_size,
                  const VMMZPETable *pe_table,
                  const VMMZProcessTable *process_table,
                  VMMZCopyTable *copy_table,
                  const QccString predicted_vmem_filename);


#endif /* VMMZ_PE_H */
