/*
 *
 * VMMZ: Delta Encoding of Virtual-Machine Memory
 * Copyright (C) 2015-2016  James E. Fowler
 *
 * This file is a part of VMMZ.
 *
 * VMMZ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VMMZ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "vmmz.h"


void VMMZProcessTableInitialize(VMMZProcessTable *process_table)
{
  if (process_table != NULL)
    {
      process_table->page_tables = NULL;
      process_table->num_processes = 0;
    }
}


int VMMZProcessTableAlloc(VMMZProcessTable *process_table)
{
  int index;
  
  if (process_table != NULL)
    if (process_table->num_processes > 0)
      {
        if ((process_table->page_tables =
             (VMMZPageTable *)malloc(sizeof(VMMZPageTable) *
                                     process_table->num_processes)) == NULL)
          {
            QccErrorAddMessage("(VMMZProcessTableAlloc): Error allocating memory");
            return(1);
          }
        for (index = 0; index < process_table->num_processes; index++)
          VMMZPageTableInitialize(&process_table->page_tables[index]);
      }

  return(0);
}


void VMMZProcessTableFree(VMMZProcessTable *process_table)
{
  int index;
  
  if (process_table != NULL)
    {
      if (process_table->page_tables != NULL)
        {
          for (index = 0; index < process_table->num_processes; index++)
            VMMZPageTableFree(&process_table->page_tables[index]);
          free(process_table->page_tables);
        }
    }
}


