/*
 *
 * VMMZ: Delta Encoding of Virtual-Machine Memory
 * Copyright (C) 2015-2016  James E. Fowler
 *
 * This file is a part of VMMZ.
 *
 * VMMZ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VMMZ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "vmmz.h"


static int VMMZMountVMDKCheckLoop()
{
  QccString cmd;

  if (!strnlen(VMMZ_LOSETUP_CMD_STR, QCCSTRINGLEN))
    return(0);
  
  QccStringSprintf(cmd, "%s -f 1>/dev/null 2>/dev/null",
                   VMMZ_LOSETUP_CMD_STR);

  if (system(cmd) != 0)
    return(1);

  return(0);
}


int VMMZMountVMDK(const QccString vmdk_filename,
                  QccString mount_point)
{
  int return_value = 0;
  QccString cmd;

  if (!QccFileExists(vmdk_filename))
    {
      QccErrorAddMessage("(VMMZMountVMDK): file %s does not exist",
                         vmdk_filename);
      goto Error;
    }
  
  if (VMMZMountVMDKCheckLoop())
    {
      QccErrorAddMessage("(VMMZMountVMDK): No loop device found. Try running 'losetup -f' as root");
      goto Error;
    }
    
  if (mkdtemp(mount_point) == NULL)
    {
      QccErrorAddMessage("(VMMZMountVMDK): Cannot create temporary mount point (%s)", mount_point);
      goto Error;
    }

  QccStringSprintf(cmd, "%s -r %s %s 2>/dev/null",
                   VMMZ_VMWARE_MOUNT_CMD_STR,
                   vmdk_filename,
                   mount_point);
  
  if (system(cmd) != 0)
    {
      QccErrorAddMessage("(VMMZMountVMDK): Error mounting VMDK file (%s)",
                         vmdk_filename);
      goto Error;
    }

  return_value = 0;
  goto Return;
 Error:
  return_value = 1;
 Return:
  return(return_value);
}


int VMMZUnmountVMDK(const QccString mount_point)
{
  QccString cmd;

  if (!QccStringNull(mount_point))
    {
      QccStringSprintf(cmd, "%s -d %s 2>/dev/null",
                       VMMZ_VMWARE_MOUNT_CMD_STR,
                       mount_point);
      
      system(cmd);
    }
  
  return(0);
}

