/*
 *
 * VMMZ: Delta Encoding of Virtual-Machine Memory
 * Copyright (C) 2015-2016  James E. Fowler
 *
 * This file is a part of VMMZ.
 *
 * VMMZ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VMMZ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "vmmzdiff.h"

#define USG_STRING "[-q %:] [-m %: %s:mapfile] %s:vmemfile1 %s:vmemfile2"

int Quiet = 0;
QccString Filename1;
QccString Filename2;
int MapSpecified = 0;
QccIMGImage OutputImage;

int main(int argc, char *argv[])
{
  long filesize;
  long num_differences;
  
  QccInit(argc, argv);
  
  QccIMGImageInitialize(&OutputImage);
  
  VMMZSetHeader();

  if (QccParseParameters(argc, argv,
                         USG_STRING,
                         &Quiet,
                         &MapSpecified,
                         OutputImage.filename,
                         Filename1,
                         Filename2))
    QccErrorExit();
  
  if (VMMZDiff(Filename1,
               Filename2,
               &filesize,
               &num_differences,
               ((MapSpecified) ? &OutputImage : NULL)))
    {
      QccErrorAddMessage("%s: Error calling VMMZDiff()",
                         argv[0]);
      QccErrorExit();
    }

  if (MapSpecified)
    if (QccIMGImageWrite(&OutputImage))
      {
        QccErrorAddMessage("%s: Error calling QccIMGImageWrite()",
                           argv[0]);
        QccErrorExit();
      }
      

  if (!Quiet)
    {
      printf("  Filesize: %ld\n", filesize);
      printf("  Num of differences: %ld (%0.2f%%)\n",
             num_differences,
             ((double)num_differences) / filesize * 100);
    }

  QccIMGImageFree(&OutputImage);
  
  QccExit;
}
