/*
 *
 * VMMZ: Delta Encoding of Virtual-Machine Memory
 * Copyright (C) 2015-2016  James E. Fowler
 *
 * This file is a part of VMMZ.
 *
 * VMMZ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VMMZ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "vmmzencode.h"

#define USG_STRING "[-q %:] [-d %: %s:vmdk_file] [-vp %: %s:volatility_profile] [-p %: %s:predicted_vmem] %s:baseline_vmem %s:current_vmem %s:bitstream"

int Quiet = 0;
int VMDKSpecified = 0;
QccString VMDKFilename;
int VolatilityProfileSpecified = 0;
QccString VolatilityProfile;
int PredictedMemorySpecified = 0;
QccString PredictedMemoryFilename;
QccString BaselineFilename;
QccString CurrentMemoryFilename;
QccString OutfileFilename;

int main(int argc, char *argv[])
{
  long filesize1 = 0;
  long filesize2 = 0;
  double rate;
  double compression_ratio;
  clock_t time1;
  clock_t time2;

  time1 = clock();
  
  QccInit(argc, argv);
  
  VMMZSetHeader();

  if (QccParseParameters(argc, argv,
                         USG_STRING,
                         &Quiet,
                         &VMDKSpecified,
                         VMDKFilename,
                         &VolatilityProfileSpecified,
                         VolatilityProfile,
                         &PredictedMemorySpecified,
                         PredictedMemoryFilename,
                         BaselineFilename,
                         CurrentMemoryFilename,
                         OutfileFilename))
    QccErrorExit();
  
  if (!VMDKSpecified)
    QccConvertToQccString(VMDKFilename, VMMZ_VMWARE_DEFAULT_VMDK_STR);
  
  if (!VolatilityProfileSpecified)
    QccConvertToQccString(VolatilityProfile,
                          VMMZ_VOLATILITY_DEFAULT_PROFILE_STR);
  
  if (VMMZEncode(BaselineFilename,
                 CurrentMemoryFilename,
                 OutfileFilename,
                 VMDKFilename,
                 VolatilityProfile,
                 &filesize1,
                 ((PredictedMemorySpecified) ? PredictedMemoryFilename : NULL)))
    {
      QccErrorAddMessage("%s: Error calling VMMZEncode()",
                         argv[0]);
      QccErrorExit();
    }
  
  time2 = clock();
  
  if (!Quiet)
    {
      if (QccFileGetSize(OutfileFilename,
                         NULL,
                         &filesize2))
        {
          QccErrorAddMessage("%s: Error calling QccFileGetSize()",
                             argv[0]);
          QccErrorExit();
        }

      rate = ((double)filesize2) * 8 / filesize1;
      compression_ratio = 8 / rate;

      printf("      Original size: %ld bytes\n", filesize1);
      printf("    Compressed size: %ld bytes (%0.1f%%)\n",
             filesize2,
             100 - ((double)filesize2) / filesize1 * 100);
      printf("               Rate: %0.4f bits/byte\n", rate);
      printf("  Compression ratio: %0.2f : 1\n", compression_ratio);
      printf("       Elapsed time: %0.2f secs\n",
             (double)(time2 - time1) / CLOCKS_PER_SEC);
    }
    
  QccExit;
}
